@@ -1996,6 +1996,7 @@ sub X_PROTOCOL_REVISION () {0}
                 'height'=> 500,
                 'parent'=> 0,
                 'mask'  => StructureNotifyMask,
+		'cmap' => 1,
                 'attributes'=> [GLX_RGBA],
         );
 sub glpOpenWindow {
@@ -2009,7 +2010,7 @@ sub glpOpenWindow {
                 $p{$k} = $a{$k};
         }
         glpcOpenWindow($p{x},$p{y},$p{width},$p{height},
-                       $p{parent},$p{'mask'},
+                       $p{parent},$p{'mask'},$p{'cmap'},
                        @{$p{attributes}});
 }
 1;
