@@ -34,7 +34,16 @@ require 'VRMLFields.pm';
 require 'VRMLNodes.pm';
 require 'VRMLRend.pm';
 
-# XXX Texture coordinates
+#######################################################################
+#######################################################################
+#######################################################################
+#
+# RendRay --
+#  code for checking whether a ray (defined by mouse pointer)
+#  intersects with the geometry of the primitive.
+#
+#
+
 %RendRayC = (
 Box => '
 	float x = $f(size,0)/2;
@@ -330,6 +339,16 @@ IndexedFaceSet => '
 
 );
 
+#####################################################################3
+#####################################################################3
+#####################################################################3
+#
+# GenPolyRep
+#  code for generating internal polygonal representations
+#  of some nodes (ElevationGrid, Extrusion and IndexedFaceSet)
+#
+#
+
 
 # In one sense, we could just plot the polygons here and be done
 # with it -- displaylists would speed it up.
@@ -656,6 +675,14 @@ IndexedFaceSet => '
 ',
 );
 
+######################################################################
+######################################################################
+######################################################################
+#
+# Get3
+#  get a coordinate / color / normal array from the node.
+#
+
 %Get3C = (
 Coordinate => '
 	*n = $f_n(point); 
@@ -671,6 +698,15 @@ Normal => '
 '
 );
 
+######################################################################
+######################################################################
+######################################################################
+#
+# Generation
+#  Functions for generating the code
+#
+
+
 {
 	my %AllNodes = (%RendC, %PrepC, %FinC, %ChildC, %Get3C, %LightC);
 	@NodeTypes = keys %AllNodes;
@@ -782,7 +818,7 @@ CODE:
 	}
 	$o .= "\t*ptr_++ = sizeof(struct VRML_$name);\n";
 	$o .= "RETVAL=&(virt_${name});
-	printf(\"$name virtual: %d\\n\", RETVAL);
+	if(verbose) printf(\"$name virtual: %d\\n\", RETVAL);
 OUTPUT:
 	RETVAL
 ";
@@ -887,7 +923,7 @@ static struct VRML_Virt virt_${n} = { ".
 		} else {
 			$f .= "void ${n}_$_(void *nod_)";
 		}
-		$f .= "{
+		$f .= "{ /* GENERATED FROM HASH ${$_}C, MEMBER $n */
 			struct VRML_$n *this_ = (struct VRML_$n *)nod_;
 			{$c}
 			}";
@@ -896,6 +932,14 @@ static struct VRML_Virt virt_${n} = { ".
 	return ($f,$v);
 }
 
+######################################################################
+######################################################################
+######################################################################
+#
+# gen - the main function. this contains much verbatim code
+#
+#
+
 sub gen {
 	for(@VRML::Fields) {
 		push @str, ("VRML::Field::$_")->cstruct;
@@ -914,6 +958,9 @@ sub gen {
 	open XS, ">VRMLFunc.xs";
 	print XS '
 /* VRMLFunc.c generated by VRMLC.pm. DO NOT MODIFY, MODIFY VRMLC.pm INSTEAD */
+
+/* Code here comes almost verbatim from VRMLC.pm */
+
 #include "EXTERN.h"
 #include "perl.h"
 #include "XSUB.h"
@@ -1023,8 +1070,8 @@ GLdouble projMatrix[16];
 /* defines for raycasting: */
 #define APPROX(a,b) (fabs(a-b)<0.00000001)
 #define XEQ (APPROX(t_r1.x,t_r2.x))
-#define YEQ (APPROX(t_r1.x,t_r2.x))
-#define ZEQ (APPROX(t_r1.x,t_r2.x))
+#define YEQ (APPROX(t_r1.y,t_r2.y))
+#define ZEQ (APPROX(t_r1.z,t_r2.z))
 /* xrat(a) = ratio to reach coordinate a on axis x */
 #define XRAT(a) (((a)-t_r1.x)/(t_r2.x-t_r1.x))
 #define YRAT(a) (((a)-t_r1.y)/(t_r2.y-t_r1.y))
@@ -1054,8 +1101,11 @@ float tx,float ty, char *descr)  {
 	GLdouble projMatrix[16];
 	GLdouble wx, wy, wz;
 	/* Real rat-testing */
-	if(verbose) printf("RAY HIT %s! %f (%f %f %f) (%f %f %f)\n",
-		descr, rat,cx,cy,cz,nx,ny,nz);
+	if(verbose) printf("RAY HIT %s! %f (%f %f %f) (%f %f %f)\nR: (%f %f %f) (%f %f %f)\n",
+		descr, rat,cx,cy,cz,nx,ny,nz,
+		t_r1.x, t_r1.y, t_r1.z,
+		t_r2.x, t_r2.y, t_r2.z
+		);
 	if(rat<0 || (rat>hpdist && hpdist >= 0)) {
 		return;
 	}
@@ -1095,6 +1145,10 @@ void regen_polyrep(void *node) ;
 void calc_poly_normals_flat(struct VRML_PolyRep *rep);
 void render_ray_polyrep(void *node,
 	int npoints, struct SFColor *points);
+
+/*********************************************************************
+ * Code here is generated from the hashes in VRMLC.pm and VRMLRend.pm
+ */
 	';
 
 #######################################################j
@@ -1105,6 +1159,17 @@ void render_ray_polyrep(void *node,
 #######################################################
 	print XS <<'ENDHERE'
 
+/*********************************************************************
+ * Code here again comes almost verbatim from VRMLC.pm 
+ */
+
+/*********************************************************************
+ *********************************************************************
+ *
+ * render_polyrep : render one of the internal polygonal representations
+ * for some nodes
+ */
+ 
 
 void render_polyrep(void *node, 
 	int npoints, struct SFColor *points,
@@ -1166,6 +1231,13 @@ void render_polyrep(void *node,
 	glEnd();
 }
 
+/*********************************************************************
+ *********************************************************************
+ *
+ * render_ray_polyrep : get intersections of a ray with one of the
+ * polygonal representations
+ */
+
 void render_ray_polyrep(void *node,
 	int npoints, struct SFColor *points)
 {
@@ -1344,6 +1416,13 @@ void calc_poly_normals_flat(struct VRML_PolyRep *rep)
 	}
 }
 
+/*********************************************************************
+ *********************************************************************
+ *
+ * render_node : call the correct virtual functions to render the node
+ * depending on what we are doing right now.
+ */
+
 void render_node(void *node) {
 	struct VRML_Virt *v;
 	struct VRML_Box *p;
