@@ -4,6 +4,15 @@
 # for conditions of use and redistribution.
 
 
+#######################################################################
+#######################################################################
+#######################################################################
+#
+# Rend --
+#  actually render the node
+#
+#
+
 # Rend = real rendering
 %RendC = (
 # XXX Tex coords wrong :(
@@ -648,8 +657,16 @@ Background => '
 
 );
 
-# Prep = prepare for rendering children - if searching for nodes,
-# unnecessary
+#######################################################################
+#######################################################################
+#######################################################################
+#
+# Prep --
+#  Prepare for rendering a node - e.g. for transforms, do the transform
+#  but not the children.
+#
+#
+
 %PrepC = (
 Transform => (join '','
 	glPushMatrix();
@@ -781,7 +798,8 @@ Viewpoint => (join '','
 			a1, angle);
 
 		glMatrixMode(GL_PROJECTION);
-		glLoadIdentity();
+		glPopMatrix(); /* This is so we do picking right */
+		/* glLoadIdentity(); */
 		gluPerspective(angle,vp[2]/(float)vp[3],0.1,200000);
 		glMatrixMode(GL_MODELVIEW);
 	}
@@ -789,6 +807,16 @@ Viewpoint => (join '','
 
 );
 
+#######################################################################
+#######################################################################
+#######################################################################
+#
+# Fin --
+#  Finish the rendering i.e. restore matrices and whatever to the
+#  original state.
+#
+#
+
 # Finish rendering
 %FinC = (
 Transform => (join '','
@@ -799,6 +827,15 @@ Billboard => (join '','
 '),
 );
 
+#######################################################################
+#######################################################################
+#######################################################################
+#
+# Child --
+#  Render the actual children of the node.
+#
+#
+
 # Render children (real child nodes, not e.g. appearance/geometry)
 %ChildC = (
 	Group => '
@@ -878,6 +915,15 @@ $ChildC{Transform} = $ChildC{Group};
 $ChildC{Billboard} = $ChildC{Group};
 $ChildC{Anchor} = $ChildC{Group};
 
+#######################################################################
+#######################################################################
+#######################################################################
+#
+# Light --
+#  Render a light. XXX This needs work to be like the spec :(
+#
+#
+
 # NO startlist -- nextlight() may change :(
 %LightC = (
 	DirectionalLight => '
